function res = catfr_fr_stats()
%CATFR_FR_STATS run statistical analysis for catfr/fr comparison
%
% This performs a number of statistical analyses reported in the
% manuscript, primarily all ANOVAs reported in the results section.
%
% The ordering of the results in this function and the output
% structure match the ordering of the results in the manuscript.

%% behavioral results

% spc anova
spc_table = readtable('data/spc_data.csv');

% do some formatting
experiments = {'FR1','catFR1'};
cnt=1;
for e = 1:2
    for s = 1:12
        new_table(cnt,:) = spc_table.prec(spc_table.serialpos == s-1 & strcmp(spc_table.experiment,experiments{e}));
        varNames1{cnt} = ['Y' num2str(cnt)];
        ex{cnt} = experiments{e};
        sp{cnt} = num2str(s);
        cnt=cnt+1;
    end 
end

new_table = array2table(new_table','VariableNames',varNames1);

factorNames = {'serialpos','list'};
within = table(sp',...
    ex','VariableNames',factorNames);
within.serialpos = categorical(within.serialpos);

rm = fitrm(new_table,'Y1-Y24~1', 'WithinDesign', within);
res.spc_anova = ranova(rm, 'WithinModel','serialpos*list');


%% encoding and retrieval success

% encoding
enc_cc = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/encoding/catFR1');
enc_cu = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/encoding/catFR1toFR1');
enc_uu = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/encoding/FR1');
enc_uc = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/encoding/FR1toCatFR1');

% retrieval
rec_cc = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/retrieval/catFR1');
rec_cu = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/retrieval/catFR1toFR1');
rec_uu = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/retrieval/FR1');
rec_uc = h5read('data/aucsubj_comps.hdf5','/aucs_scores/raw/retrieval/FR1toCatFR1'); 

% decoding results
[~, p, ~, st] = ttest([enc_cc, enc_uu, rec_cc, rec_uu], .5);

res.success_decode.classifier = {'cat_enc', ...
                                 'unr_enc', ...
                                 'cat_rec', ...
                                 'unr_enc'};
res.success_decode.p = p;
res.success_decode.t = st.tstat;
res.success_decode.se = st.sd/sqrt(length(enc_cc));

% cross-decoding results
[~, p, ~, st] = ttest([enc_cu, enc_uc, rec_cu, rec_uc], .5);

res.success_cross_decode.classifier = {'cat_enc', ...
                                       'unr_enc', ...
                                       'cat_rec', ...
                                       'unr_enc'};
res.success_cross_decode.p = p;
res.success_cross_decode.t = st.tstat;
res.success_cross_decode.se = st.sd/sqrt(length(enc_cu));

% specificity results
[~, p, ~, st] = ttest([enc_cc - enc_cu, ...
                       enc_uu - enc_uc, ...
                       rec_cc - rec_cu, ...
                       rec_uu - rec_uc]);
                   
res.success_difference.classifier = {'cat_enc', ...
                                       'unr_enc', ...
                                       'cat_rec', ...
                                       'unr_enc'};
res.success_difference.p = p;
res.success_difference.t = st.tstat;
res.success_difference.se = st.sd/sqrt(length(enc_cc));


% anova
varNames = {'Y1','Y2','Y3','Y4'};

tbl = array2table([enc_cc - enc_cu, ...
    enc_uu-enc_uc, ...
    rec_cc-rec_cu, ...
    rec_uu-rec_uc],'VariableNames',varNames);

factorNames = {'train', 'phase'};
within = table( {'C';'U';'C';'U'},...
    {'E';'E';'R';'R'},'VariableNames',factorNames);

rm = fitrm(tbl,'Y1-Y4~1', 'WithinDesign', within);
res.success_anova = ranova(rm, 'WithinModel','train*phase');


%% temporal and category clustering analyses

enc =  h5read('data/aucsubj_scores_temporal.hdf5', ...
    '/aucs_scores/raw/encoding/withinvsbetweenFR1');
subjects_enc =  h5read('data/aucsubj_scores_temporal.hdf5', ...
    '/subjects/raw/encoding/withinvsbetweenFR1');
rec = h5read('data/aucsubj_scores_temporal.hdf5', ...
    '/aucs_scores/raw/retrieval/withinvsbetweenFR1');
subjects_rec = h5read('data/aucsubj_scores_temporal.hdf5', ...
    '/subjects/raw/retrieval/withinvsbetweenFR1');

subjects_enc = deblank(subjects_enc);
subjects_rec = deblank(subjects_rec);

enc_within_between = h5read('data/aucsubj_scores.hdf5', ...
    '/aucs_scores/raw/encoding/withinvsbetweenCatFR1');
subjects_enc_cat =  h5read('data/aucsubj_scores.hdf5', ...
    '/subjects/raw/encoding/withinvsbetweenCatFR1');
rec_within_between = h5read('data/aucsubj_scores.hdf5', ...
    '/aucs_scores/raw/retrieval/withinvsbetweenCatFR1');
subjects_rec_cat =  h5read('data/aucsubj_scores.hdf5', ...
    '/subjects/raw/retrieval/withinvsbetweenCatFR1');

subjects_enc_cat = deblank(subjects_enc_cat);
subjects_rec_cat = deblank(subjects_rec_cat);

subs = mintersect(subjects_enc, ...
                  subjects_rec, ...
                  subjects_enc_cat, ...
                  subjects_rec_cat);

ia = ismember(subjects_enc, subs);
ib = ismember(subjects_rec, subs);
ic = ismember(subjects_enc_cat, subs);
id = ismember(subjects_rec_cat, subs);

% cluster decoding results

y = [enc(ia) rec(ib) enc_within_between(ic) rec_within_between(id)];

[~, p, ~, st] = ttest(y, 0.5);

res.cluster_decode.classifier = {'temporal_enc', ...
                                 'temporal_rec', ...
                                 'categorical_enc', ...
                                 'categorical_enc'};
res.cluster_decode.p = p;
res.cluster_decode.t = st.tstat;
res.cluster_decode.se = st.sd/sqrt(length(enc_cu));

% category clusterin enc vs. ret

[~, p, ~, st] = ttest(y(:,3), y(:,4));

res.category_clust_diff.p = p;
res.category_clust_diff.t = st.tstat;
res.category_clust_diff.se = st.sd/sqrt(size(y,1));

% cluster decoding anova

varNames = {'Y1','Y2','Y3','Y4'};
tbl = array2table(y,'VariableNames',varNames);
factorNames = {'classifier','phase'};
within = table({'T','T','C','C'}',...
{'E','R','E','R'}','VariableNames',factorNames);
% to confirm post-hoc tests
within.phase = categorical(within.phase);
within.classifier = categorical(within.classifier);

rm = fitrm(tbl,'Y1-Y4~1', 'WithinDesign', within);
res.cluster_anova = ranova(rm, 'WithinModel','classifier*phase');

% cross-decoding results

% cat to lag

enc_cat2lag =  h5read('data/aucsubj_LagCat_scores.hdf5','/aucs_scores/raw/encoding/cat2lag');
subjects_enc_cat2lag =  h5read('data/aucsubj_LagCat_scores.hdf5','/subjects/raw/encoding/cat2lag');
rec_cat2lag = h5read('data/aucsubj_LagCat_scores.hdf5','/aucs_scores/raw/retrieval/cat2lag');
subjects_rec_cat2lag = h5read('data/aucsubj_LagCat_scores.hdf5','/subjects/raw/retrieval/cat2lag');

subjects_enc_cat2lag = deblank(subjects_enc_cat2lag);
subjects_rec_cat2lag = deblank(subjects_rec_cat2lag);

% lag to cat

enc_lag2cat =  h5read('data/aucsubj_LagCat_scores.hdf5','/aucs_scores/raw/encoding/lag2cat');
subjects_enc_lag2cat =  h5read('data/aucsubj_LagCat_scores.hdf5','/subjects/raw/encoding/lag2cat');
rec_lag2cat = h5read('data/aucsubj_LagCat_scores.hdf5','/aucs_scores/raw/retrieval/lag2cat');
subjects_rec_lag2cat = h5read('data/aucsubj_LagCat_scores.hdf5','/subjects/raw/retrieval/lag2cat');

subjects_enc_lag2cat = deblank(subjects_enc_lag2cat);
subjects_rec_lag2cat = deblank(subjects_rec_lag2cat);

ia2 = ismember(subjects_enc_cat2lag, subs);
ib2 = ismember(subjects_rec_cat2lag, subs);
ic2 = ismember(subjects_enc_lag2cat, subs);
id2 = ismember(subjects_rec_lag2cat, subs);

y2 = [enc_cat2lag(ia2) rec_cat2lag(ib2) enc_lag2cat(ic2) rec_lag2cat(id2)];

Y = y-y2; % decoding - cross-decoding

[~, p, ~, st] = ttest(Y);

res.cluster_diff.classifier = {'temporal_enc', ...
                               'temporal_rec', ...
                               'categorical_enc', ...
                               'categorical_enc'};

res.cluster_diff.p = p;
res.cluster_diff.t = st.tstat;
res.cluster_diff.se = st.sd/sqrt(size(Y,1));

% decoding - cross-decoding anova

varNames = {'Y1','Y2','Y3','Y4'};
tbl = array2table(Y,'VariableNames',varNames);
factorNames = {'classifier','phase'};
within = table({'T','T','C','C'}',...
{'E','R','E','R'}','VariableNames',factorNames);
% to confirm post-hoc tests
within.phase = categorical(within.phase);
within.classifier = categorical(within.classifier);

rm = fitrm(tbl,'Y1-Y4~1', 'WithinDesign', within);
res.cluster_diff_anova = ranova(rm, 'WithinModel','classifier*phase');

[~,p,~,st] = ttest(Y(:,3), Y(:,4));

res.category_clust_specificity_diff.p = p;
res.category_clust_specificity_diff.t = st.tstat;
res.category_clust_specificity_diff.se = st.sd/sqrt(size(y,1));


end

function runIntersect = mintersect(varargin)
%MINTERSECT Multiple set intersection.
%   MINTERSECT(A,B,C,...) when A,B,C... are vectors returns the values 
%   common to all A,B,C... The result will be sorted.  A,B,C... can be cell
%   arrays of strings.  
%
%   MINTERSECT repeatedly evaluates INTERSECT on successive pairs of sets, 
%   which may not be very efficient.  For a large number of sets, this should
%   probably be reimplemented using some kind of tree algorithm.
%
%   MINTERSECT(A,B,'rows') when A,B,C... are matrices with the same
%   number of columns returns the rows common to all A,B,C...
%
%   See also INTERSECT

flag = 0;
if isempty(varargin)
    error('No inputs specified.')
else
    if isequal(varargin{end},'rows')
        flag = 'rows';
        setArray = varargin(1:end-1);
    else
        setArray = varargin;
    end
end

runIntersect = setArray{1};
for i = 2:length(setArray)
    
    if isequal(flag,'rows')
        runIntersect = intersect(runIntersect,setArray{i},'rows');
    elseif flag == 0
        runIntersect = intersect(runIntersect,setArray{i});
    else 
        error('Flag not set.')
    end
    
    if isempty(runIntersect)
        return
    end
    
end

end

